%Description: create terramechanics lookup tables over range of input conditions
%inputs: 
    % Wheel parameters, Terrain parameters, range of wheel slip, range of front contact angle, range of unloading sinkage
%outputs:
    % tables: normal load, thrust force, resistance force
    % input ranges: normal force, contact angle, unloading sinkage

function [PosNormalLookup,PosForceLookup, PosResistLookup, NormalForceRange, ContactAngleLookup, MaxSinkageLookup] = buildTable(Wheel,Terrain,slip_range,thetaf_range,unloadsink_range)

%% Initialize Tables

PosNormalLookup = zeros(length(thetaf_range),length(slip_range),length(unloadsink_range));
PosForceLookup = zeros(length(thetaf_range),length(slip_range),length(unloadsink_range));
PosResistLookup = zeros(length(thetaf_range),length(slip_range),length(unloadsink_range));
MaxSinkageLookup = zeros(length(thetaf_range),length(unloadsink_range));

%% Build Lookup Table 

    %recall, for Wy*r>Vx slip = (Wy*r-Vx)/(Wy*r)
            %for Wy*r<Vx slip = (Wy*r-Vx)/(Vx)          


for thetaf_index = 1:length(thetaf_range)
thetaf=thetaf_range(thetaf_index);
% disp(['theta_f = ',num2str(thetaf*(180/pi))]);
   
    for slip_index=1:length(slip_range)
        slip=slip_range(slip_index); %wheel slip
        
        for unloadsink_index=1:length(unloadsink_range)
            unloadsink=unloadsink_range(unloadsink_index);
        
            [Ntemp,Ftemp,Rtemp,Ttemp,MaxSinktemp] = calculateForceTorque(Wheel,Terrain,thetaf,slip,unloadsink);

            if isnan(Ntemp) 
                PosNormalLookup(thetaf_index,slip_index,unloadsink_index)=0;
            else
                PosNormalLookup(thetaf_index,slip_index,unloadsink_index)=Ntemp;
            end

    %         if isnan(Ttemp)
    %             PosTorqueLookup(Index)=0;
    %         else
    %         PosTorqueLookup(Index)=Ttemp;
    %         end
    %         
            if isnan(Ftemp)
                PosForceLookup(thetaf_index,slip_index,unloadsink_index)=0;
            else
                PosForceLookup(thetaf_index,slip_index,unloadsink_index)=Ftemp;
            end

            if isnan(Rtemp)
                PosResistLookup(thetaf_index,slip_index,unloadsink_index)=0;
            else
                PosResistLookup(thetaf_index,slip_index,unloadsink_index)=Rtemp;
            end        

            if isnan(MaxSinktemp)
                MaxSinkageLookup(thetaf_index,unloadsink_index)=0;
            else
                MaxSinkageLookup(thetaf_index,unloadsink_index)=MaxSinktemp;
            end    
            
        end
        
    end


end


%% Flip Normal Lookup to make Contact Angle Lookup
%% impossible to flip if non monotonic!!


%Flip to create sparse matrix
PosNormalLookup(PosNormalLookup<0)=0;
temp_min=min(min(min(PosNormalLookup))); temp_max=max(max(max(PosNormalLookup)));
NormalForceRange=linspace(temp_min,temp_max,250);

ContactAngleLookup=nan(length(NormalForceRange),length(slip_range),length(unloadsink_range));

for index1=1:length(slip_range)
for index2=1:length(thetaf_range)
for index3=1:length(unloadsink_range)
    Normal=PosNormalLookup(index2,index1,index3);
    Normal_rounded=interp1(NormalForceRange,NormalForceRange,Normal,'nearest');
    ContactAngleLookup(NormalForceRange==Normal_rounded,index1,index3)=thetaf_range(index2);
end
end
end
ContactAngleLookup(ContactAngleLookup==0)=nan;

Temp_ContactAngleLookup = nan(size(ContactAngleLookup));

for unloadsink_index=1:length(unloadsink_range)

%find real values for negative and positive slip regions
[rows_neg,clms_neg]=find(~isnan(ContactAngleLookup(:,slip_range<0,unloadsink_index)));
[rows_pos,clms_pos]=find(~isnan(ContactAngleLookup(:,slip_range>=0,unloadsink_index)));
clms_pos=clms_pos+max(clms_neg);

%make x,y,z matrices
SparceY_neg = NormalForceRange(rows_neg);
SparceX_neg = slip_range(clms_neg);

SparceY_pos = NormalForceRange(rows_pos);
SparceX_pos = slip_range(clms_pos);

SparceZ_neg=nan(length(rows_neg),1);
SparceZ_pos=nan(length(rows_pos),1);
for index=1:length(rows_neg)
SparceZ_neg(index,1) = ContactAngleLookup(rows_neg(index),clms_neg(index),unloadsink_index);
end
for index=1:length(rows_pos)
SparceZ_pos(index,1) = ContactAngleLookup(rows_pos(index),clms_pos(index),unloadsink_index);
end


% Yrange=linspace(min([SparceY_neg,SparceY_pos]),max([SparceY_neg,SparceY_pos]),100);
Yrange=NormalForceRange;
[X_neg,Y_neg] = meshgrid(SparceX_neg,Yrange);
[X_pos,Y_pos] = meshgrid(SparceX_pos,Yrange);

Z_neg = griddata(SparceX_neg,SparceY_neg,SparceZ_neg,X_neg,Y_neg,'cubic');
Z_pos = griddata(SparceX_pos,SparceY_pos,SparceZ_pos,X_pos,Y_pos,'cubic');

%remove unnecessary rows/columns
X=[X_neg(1,:),X_pos(1,:)];
% Y=Y_pos(:,1);
Z=[Z_neg,Z_pos];

%remove repetitive values of slip
xunique=nan(length(slip_range),1);
for index=1:length(slip_range)
xunique(index)=find(X==slip_range(index),1,'first');
end

%output
% X=X(xunique); %should equal slip_range
Temp_ContactAngleLookup(:,:,unloadsink_index)=Z(:,xunique);
% NormalForceRange=Y; %should equal normal force range

end

ContactAngleLookup=Temp_ContactAngleLookup;


end