%Description: perform terramechanics calculations
%inputs:
    % Wheel parameters, Terrain parameters, wheel slip, front contact angle, unloading sinkage
%outputs:
    % normal load, thrust force, resistance force, torque, unloading sinkage


%%
%%% Citation %%%
% [1]: Wong, J.Y. Terramechanics and off-road vehicle engineering. Terrain
% behaviour, off-road vehicle performance and design. Second Edition.
% Elsevier, 2010.

% [2]: Calvin (HyukChul) Nho, Jon R. Salton, Barry Spletzer. Mobility 
% Analysis Tool Based on the Fundamental Principle of Conservation of 
% Energy. SANDIA REPORT SAND2007-2791, 2007.

% [3]: Wong, J.Y., and Reece, A.R. (1967). Prediction of rigid wheel
% performance based on the analysis of soil-wheel stresses - Part II.
% Performance of towed rigid wheels. Journal of Terramechanics, 4(2), 7-25.

% [4]: Wong, J.Y., and Reece, A.R. (1967). Prediction of rigid wheel
% performance based on the analysis of soil-wheel stresses - Part II.
% Performance of towed rigid wheels. Journal of Terramechanics, 4(1), 81-98.

%%sinkage: z=(1-cos(theta_f))*Wheel.r %[4], p.9

function [Normal_Load,Thrust_Force,Resistance_Force,Torque,Max_GroundSinkage] = calculateForceTorque(Wheel,Terrain,theta_f,slip,sink_unload)

%% addition made for tandem wheels (repetitive loading)
%%as wheel in front pass over terrain, the ground sinks
    %the maximum value will determine pressure build up in the soil
switch nargin
    case 4 %thetaf_prior not specified
        sink_unload=0;
end

%calculate sinkage_zero (sinkage where pressure is zero - settling of
%ground), [1] p.87
if ~sink_unload
    sink_zero=0; %sink_zero equals zero for first wheel
else
    sink_zero = sink_unload-(((Terrain.kc/Wheel.b)+Terrain.kphi)*sink_unload^Terrain.n)/(Terrain.k0+Terrain.Au*sink_unload);
    sink_zero = max(sink_zero,0); %negative sinkage not possible
end


%maximum ground sinkage:
% disp([num2str((1-cos(theta_f))*Wheel.r+sink_zero),'    ',num2str(sink_unload)]);
Max_GroundSinkage = max((1-cos(theta_f))*Wheel.r+sink_zero,sink_unload);

%%
%%calculate location of maximum stress
theta_m = calculate_thetam(Terrain,theta_f,slip);

%%calculate rear contact angle
theta_r = (Terrain.b0+(Terrain.b1*slip))*theta_f;

number_theta_divisions = 100;
delta_theta = (theta_f-theta_r)/number_theta_divisions;

%initialize intermediate calculations
Normal_step = zeros(number_theta_divisions+1,1);
Thrust_step = Normal_step;
Resist_step = Normal_step;
Torque_step = Normal_step;

for index = 1:number_theta_divisions+1
    theta = theta_r+(index-1)*delta_theta;

    %%calculate shear deformation and theta_eq: 
    %%front region = theta, rear region = theta_eq
    [j_shear, theta_eq] = calculate_jshear_thetaeq(Wheel,slip,theta_r,theta_m,theta_f,theta);

    %%calculate normal and shear stresses
    sigma_tip = calculate_sigmatip(Wheel,Terrain,theta_r,theta_f,theta_eq,sink_unload,sink_zero);
    sigma_concave = calculate_sigmaconcave(Wheel,Terrain,theta_r,theta_f,theta_eq,sink_unload,sink_zero);
    sigma = calculate_sigma(Wheel,sigma_tip,sigma_concave);

    tau = calculate_tau(Terrain,j_shear,sigma,slip);
    
    Normal_step(index) = sigma*cos(theta)+tau*sin(theta); %[1] p.409
    Thrust_step(index) = tau*cos(theta); %[1] p.410
    Resist_step(index) = sigma*sin(theta); %[1] p.410
    Torque_step(index) = tau; %[1] p.410

end
        
Normal_Load = Wheel.b*Wheel.r*trapz(Normal_step.*delta_theta); %[1] p.409
Thrust_Force = Wheel.b*Wheel.r*trapz(Thrust_step.*delta_theta); %[1] p.410
Resistance_Force = Wheel.b*Wheel.r*trapz(Resist_step.*delta_theta); %[1] p.410
Torque = Wheel.b*Wheel.r^2*trapz(Torque_step.*delta_theta); %[1] p.410
end

%% Location of maximum stress
function thetam_out = calculate_thetam(Terrain,theta_f,slip)

if slip>=0 %Accelerating case, [1] p.385
    thetam_out = (Terrain.a0+(Terrain.a1*slip))*theta_f;
    
else % Braking case, treat as towed wheel, solve eqn for theta, [1] p.389 
    % use the identity a*cosx+b*sinx = sqrt(a^2+b^2)*cos(x-atan2(b,a))
    P1=tan((pi/4)-(Terrain.phi/2));
    P2=1+slip;
    theta_temp = acos(P2/(sqrt(1+P1^2)))+atan2(-P1,1);
    thetam_out = min(theta_temp,theta_f);
    
    
end

end


%% Shear Deformation and Front/Rear Region Theta
function [jshear_out, thetaeq_out] = calculate_jshear_thetaeq(Wheel,slip,theta_r,theta_m,theta_f,theta)

if ((theta<=theta_m) && (theta>=theta_r)) %rear region       
    thetaeq_out = theta_f - ((theta-theta_r)/(theta_m-theta_r))*(theta_f-theta_m); %[4], p.5
    
    if slip>=0 %acceleration case
        jshear_out = Wheel.r*((theta_f-theta)-(1-slip)*(sin(theta_f)-sin(theta))); %[4], p.7
    
    else %braking case
        %%% check sign of slip %%%
        jshear_out = Wheel.r*((theta_m-theta)-(1-slip)*(sin(theta_m)-sin(theta))); %[3], p.9
        
    end
    
else %front region
    thetaeq_out = theta;
    
    if slip>=0 %acceleration case
        jshear_out = Wheel.r*((theta_f-theta)-(1-slip)*(sin(theta_f)-sin(theta))); %[4], p.7
    
    else %braking case
        %%% check sign of slip %%%
        jshear_out = Wheel.r*((theta_f-theta)*((1-slip)*(sin(theta_f)-theta_m)/(theta_f-theta_m))-(1-slip)*(sin(theta_f)-sin(theta))); %[3], p.9
        
    end
    
end

end


%% Normal Stress Sigma (altered for lugges and tandem wheels)

%tip region, Zhenzhong
function sigmatip_out = calculate_sigmatip(Wheel,Terrain,theta_r,theta_f,theta_eq,sink_unload,sink_zero)

    if ((theta_eq<=theta_f) && (theta_eq>=theta_r)) %between rear and front contact angle
        
        sink_temp=(cos(theta_eq)-cos(theta_f))*Wheel.r+sink_zero; %sinkage of ground, including prior wheels
        
        if sink_temp<sink_unload %ground still recovering from prior load
            sigmatip_out = ((Terrain.kc/Wheel.b)+Terrain.kphi)*sink_unload^Terrain.n - (Terrain.k0+Terrain.Au*sink_unload)*(sink_unload-sink_temp); %[1] p.87
        else %pressure is great enough to create greater sinkage than before
            sigmatip_out = ((Terrain.kc/Wheel.b)+Terrain.kphi)*(sink_temp^Terrain.n); %[4], p.5 and [1], p.77
        end
        
    else
        sigmatip_out = 0;  
    end

end

%concave region (radius - tip length), Zhenzhong
function sigmaconcave_out = calculate_sigmaconcave(Wheel,Terrain,theta_r,theta_f,theta_eq,sink_unload,sink_zero)

    if ((theta_eq<=theta_f) && (theta_eq>=theta_r)) %between rear and front contact angle

        sink_temp=(cos(theta_eq)-cos(theta_f))*(Wheel.r-Wheel.lh)+sink_zero; %sinkage of ground, including prior wheels
        
        if sink_temp<sink_unload %ground still recovering from prior load
            sigmaconcave_out = ((Terrain.kc/Wheel.b)+Terrain.kphi)*sink_unload^Terrain.n - (Terrain.k0+Terrain.Au*sink_unload)*(sink_unload-sink_temp); %[1] p.87
        else %pressure is great enough to create greater sinkage than before
            sigmaconcave_out = ((Terrain.kc/Wheel.b)+Terrain.kphi)*(sink_temp^Terrain.n); %[4], p.5 and [1], p.77
        end
    
    else
        sigmaconcave_out = 0;  
    end

end

%combine tip and concave regions, Zhenzhong
function sigma_out = calculate_sigma(Wheel,sigma_tip,sigma_concave)
    
    sigma_out = Wheel.mu*sigma_tip + (1-Wheel.mu)*sigma_concave;
end

%% Shear Stress Tau
function tau_out = calculate_tau(Terrain,j_shear,sigma,slip)

    %Mohr-Coulomb Shear Stress Failure, [1] p.31, also p.123
    tau_max = Terrain.c + sigma*tan(Terrain.phi);  
    switch Terrain.ShearType
        case 1 %[1] p.120
            tau_out_temp = sign(j_shear)*tau_max*(1-exp(-abs(j_shear)/Terrain.Kt));
            
        case 2 %[1] p.121
            tau_out_temp = sign(j_shear)*tau_max*(abs(j_shear)/Terrain.Kt)*(1-exp(-abs(j_shear)/Terrain.Kt));
            
        case 3 %[1] p.123
            tau_out_temp = sign(j_shear)*tau_max*Terrain.Kr*(1+((1/(Terrain.Kr*(1-(1/exp(1)))))-1)*exp(-abs(j_shear)/Terrain.Kt))*(1-exp(-abs(j_shear)/Terrain.Kt));
    end
    
%     %Braking Already Taken Into Account Through Jshear    
%     %Braking case, [2] p.19
%     if slip>=0
        tau_out=tau_out_temp;
%     else
%         tau_out=-tau_out_temp;
%     end
      
end