%description: calculate motor properties for given operating condition

function[Output] = calculateMassOptimalMotorScaling(temp_TrqSpd,Speed_temp,PowerEnergyBoth_temp,Requirement,Transmission,Motor,Battery)

Scale=nan(size(temp_TrqSpd,1),1);
Ireq=nan(size(temp_TrqSpd,1),1);
Vreq=nan(size(temp_TrqSpd,1),1);
MotorMass=nan(size(temp_TrqSpd,1),1);
BatteryMass=nan(size(temp_TrqSpd,1),1);


%cannot find roots (minimum values) for an array (all conditions simultaneously) 
    %can only work with each case individually as a vector
    
for opcond_index=1:size(temp_TrqSpd,1)
    Twheel=temp_TrqSpd(opcond_index,1);     %total torque to wheels [N]
    Wwheel=temp_TrqSpd(opcond_index,2);     %wheel Requirement.MinSpeed [rad/s]
    
    speed=Speed_temp(opcond_index,1);   %vehicle speed at operating condition
    PowerEnergyBoth=PowerEnergyBoth_temp(opcond_index,1);   %operating condition specs power,energy,or both
    
    
    %Scale should be NaN if Trq/Speed input equals NaN
    Skip_NaN = any(isnan([Twheel,Wwheel]));
    
    switch Skip_NaN
        case 0 %values are good (no NaNs)
            
    %First: limit scale based on motor current limits
        %Ireq = Motor_num*[(Twheel/Motor_num)/Transmission.ratio/Kt_scaled + Inl_scaled <= Inom_scaled]
        %Ireq = Twheel/(Transmission.ratio*Kt*scale^2) + Inl*scale*Motor_num <= Inom*scale*Motor_num
    
        %solve explicitly for minimum scale that meets requirement
        %scale >= (Twheel/((Inom-Inl)*Transmission.ratio*Kt*Motor_num))^(1/3)
    
        lowerlimit_scaling=(Twheel/((Motor.I_nom-Motor.I_nl)*Transmission.ratio*Motor.Kt*Motor.number))^(1/3);
    


    %Second: limit scale based on desire to limit (Mmotor+Mbattery)
        %Vreq = Wwheel*Transmission.ratio/Kv_scaled + (Ireq/Motor_num)*R_scaled
        %Vreq = Wwheel*Transmission.ratio*scale^2/Kv + (R*Ireq)/(scale*Motor_num)
    
        %Mmotor = Mmotor_orig*scale^3 (scales with volume)
        %Mbattery = max[Ireq*Vreq/Bspecificpower, Ireq*Vreq*Requirement.Endurance/(1000(speed*Bspecificenergy)] 
            %*battery mass based on power, energy
            
        %add motor and battery mass:
        %Mass = Mmotor_scaled+Mbattery_scaled;

        %take derivative, set to zero, to find minimum
        %DerivMass = diff(Mass,'scale');
        
        %use "roots" to find zeros, choose largest real,positive value

        %take coefficients of numerator to put in polynomial form:
        DerivMassPoly1=[(3*Motor.I_nl*Wwheel*Motor.Kt^2*Motor.number^2*Transmission.ratio^3 + 3*Battery.SpecificPower*Motor.Kv*Motor.Mass*Motor.Kt^2*Motor.number^2*Transmission.ratio^2),...
            0,...
            Motor.I_nl^2*Motor.Kt^2*Motor.Kv*Motor.number^2*Motor.R*Transmission.ratio^2,...
            0,...
            0,...
            - 4*Motor.I_nl*Motor.Kt*Motor.Kv*Motor.number*Motor.R*Twheel*Transmission.ratio,...
            0,...
            0,...
            - 5*Motor.Kv*Motor.R*Twheel^2];
        
        DerivMassPoly2=[(3*Requirement.Endurance*Motor.I_nl*Wwheel*Motor.Kt^2*Motor.number^2*Transmission.ratio^3 + 3000*Battery.SpecificEnergy*Motor.Kv*Motor.Mass*speed*Motor.Kt^2*Motor.number^2*Transmission.ratio^2),...
            0,...
            Requirement.Endurance*Motor.I_nl^2*Motor.Kt^2*Motor.Kv*Motor.number^2*Motor.R*Transmission.ratio^2,...
            0,...
            0,...
            - 4*Requirement.Endurance*Motor.I_nl*Motor.Kt*Motor.Kv*Motor.number*Motor.R*Twheel*Transmission.ratio,...
            0,...
            0,...
            - 5*Requirement.Endurance*Motor.Kv*Motor.R*Twheel^2];

        
            DerivMassPolyRoots1=nanmodified_roots(DerivMassPoly1);
            DerivMassPolyRoots2=nanmodified_roots(DerivMassPoly2);
            optimal_scaling(1)=max(real(DerivMassPolyRoots1));
            optimal_scaling(2)=max(real(DerivMassPolyRoots2));
            
        %optimal scaling factor equals the maximum of the three points
            %(if lowerlimit_scaling, then must select due to constraint)
            %(if optimal_scaling, then a larger motor will give better results)
        
            
        %must only consider energy requirement for battery mass if given
        %in the spec (Power/Energy/Both spec)
        
        switch PowerEnergyBoth
            case 0 %power only
                scaling_values=[lowerlimit_scaling, optimal_scaling(1)];
                
            case 1 %energy only *battery must still meet power requirement
                scaling_values=[lowerlimit_scaling, optimal_scaling(1)];
                
            case 2 %both power and energy spec
                scaling_values=[lowerlimit_scaling, optimal_scaling];
                
        end
            
            
        %scale should be NaN if any of the scaling factors are NaN
        Check_NaN=any(isnan(scaling_values));
        switch Check_NaN
            case 1 
            Scale(opcond_index)=NaN;
            case 0
            Scale(opcond_index)=max(scaling_values);
        end
        
        
        case 1 %inputs involve NaN
            
        Scale(opcond_index)=NaN;
        
    end  
    
    Ireq(opcond_index,1) = Twheel/(Transmission.ratio*Motor.Kt*Scale(opcond_index)^2) + Motor.I_nl*Scale(opcond_index)*Motor.number;
    Vreq(opcond_index,1) = Wwheel*Transmission.ratio*Scale(opcond_index)^2/Motor.Kv + (Motor.R*Ireq(opcond_index,1))/(Scale(opcond_index)*Motor.number);    
    
    
    %% Motor Mass
    MotorMass(opcond_index,1)=Motor.Mass*Scale(opcond_index)^3; %assume mass scales with volume
    
    %% Battery Mass
    bmass_power=Ireq(opcond_index,1)*Vreq(opcond_index,1)/Battery.SpecificPower;
    bmass_energy=Ireq(opcond_index,1)*Vreq(opcond_index,1)*Requirement.Endurance/(1000*(speed*Battery.SpecificEnergy));
    
    switch PowerEnergyBoth
        case 0
            BatteryMass(opcond_index,1) = bmass_power;
        case 1
            BatteryMass(opcond_index,1) = max(bmass_power,bmass_energy);
        case 2
            BatteryMass(opcond_index,1) = max(bmass_power,bmass_energy);
    end
    
end


% Ireq(:,1) = temp_TrqSpd(:,1)./(Transmission.ratio.*Motor.Kt.*Scale.^2) + Motor.I_nl.*Scale.*Motor.number;
% Vreq(:,1) = temp_TrqSpd(:,2).*Transmission.ratio.*Scale.^2./Motor.Kv + (Motor.R.*Ireq)./(Scale.*Motor.number);


% %% Motor Mass
% MotorMass=Motor.Mass.*Scale.^3; %assume mass scales with volume

% %% Battery Mass
% BatteryMass = max(Ireq.*Vreq./Battery.SpecificPower, Ireq.*Vreq.*Requirement.Endurance./(1000.*(Requirement.MinSpeed.*Battery.SpecificEnergy)));

%% Check Feasibility
Feasibility=~isnan(Scale);


%% Output
Output.Scale=Scale;
Output.Feasibility=Feasibility;
Output.Current_req=Ireq;        %[A]
Output.Voltage_req=Vreq;        %[V]
Output.Power_req=Ireq.*Vreq;    %[W]
% Output.Energy_req=
Output.Efficiency=(temp_TrqSpd(:,1).*temp_TrqSpd(:,2))./Output.Power_req;   %[-]
Output.MotorMass=MotorMass;  %[kg]
Output.BatteryMass=BatteryMass;     %[kg]
        
