%description: determine feasibility and capabilities of vehicle design for given operating conditions

function[Output] = calculateWheelForce(Vehicle,Wheel,N_temp,Dforce_temp,Soil_temp,Speed_temp,Accel_temp,Requirement)

%% Initialization

% first wheel has zero unloading sinkage
UnloadSink=zeros(size(N_temp,1),size(N_temp,2),length(Wheel.slip_range));

% contact angle output may contain nan (for infeasible inputs)
ContactAngle=zeros(size(N_temp,1),size(N_temp,2),length(Wheel.slip_range));

% All inputs considered feasible
WheelFeasibility=ones(size(Soil_temp));

% Know types of terrains
UniqueSoil_range=unique(Soil_temp);

%thrust/resist force (for all wheels, including left/right)
ThrustForce_temp=zeros(size(N_temp,1),length(Wheel.slip_range));
ResistForce_temp=zeros(size(N_temp,1),length(Wheel.slip_range));

ThrustForce=zeros(size(N_temp,1),length(Wheel.slip_range));
ResistForce=zeros(size(N_temp,1),length(Wheel.slip_range));


%% Normal Load Infeasibility
%If a normal load is <=0, then the operating point (row) is infeasible
[temp_row,temp_col]=find(N_temp<=0);
temp_row=unique(temp_row);
WheelFeasibility(temp_row,1)=0;


%% Contact Angles and Wheel Force (over entire slip range, at each point)

%for loop to move from front wheel to rear wheels 
    %(necessary with repetitive loading consideration)
for wheel_index=1:size(N_temp,2)

    %for loop to move down the column of mass and op condition
        %(necessary for interpolation, as tables change with terrain)
    for soil_index=1:length(UniqueSoil_range)
        %choose correct soil type
        soiltype_index=UniqueSoil_range(soil_index);
        
        %select points with the same soil type
        soil_select=find(Soil_temp==soiltype_index);
        
        %terrain specific: normal force range, contact angle table,
                          %unloading sinkage
        eval(['NForceRange_temp = Wheel.',Wheel.TerrainList{soiltype_index},'.NormalForceRange;']);
        eval(['CAngleLookup_temp = Wheel.',Wheel.TerrainList{soiltype_index},'.ContactAngleLookup;']);
        eval(['MaxSinkLookup_temp = Wheel.',Wheel.TerrainList{soiltype_index},'.MaxSinkageLookup;']);
        
        %terrain specific: thrust force, resistance force tables
        eval(['ThrustLookup_temp = Wheel.',Wheel.TerrainList{soiltype_index},'.PosForceLookup;']);
        eval(['ResistLookup_temp = Wheel.',Wheel.TerrainList{soiltype_index},'.PosResistLookup;']);
        
        %interpolate contact angle [radians]
        temp_xi = ones(length(soil_select),1)*Wheel.slip_range;
        temp_yi = N_temp(soil_select,wheel_index)*ones(1,length(Wheel.slip_range));
        temp_zi = squeeze(UnloadSink(soil_select,wheel_index,:));
        switch length(soil_select)
            case 1
                temp_xi = reshape(temp_xi,1,length(temp_xi));
                temp_yi = reshape(temp_yi,1,length(temp_yi));
                temp_zi = reshape(temp_zi,1,length(temp_zi));
        end
        ContactAngle(soil_select,wheel_index,:) = nanmodified_interp3(...
            Wheel.slip_range, NForceRange_temp, Wheel.unloadsink_range, CAngleLookup_temp,...
            temp_xi, temp_yi, temp_zi);
        

        %Determine unloading sinkage for next wheel [m]
        last_wheel = (wheel_index==size(N_temp,2));
        switch last_wheel
            case 0
                UnloadSink(soil_select,wheel_index+1,:)=nanmodified_interp2(Wheel.unloadsink_range, Wheel.thetaf_range, MaxSinkLookup_temp,...
                    UnloadSink(soil_select,wheel_index,:), ContactAngle(soil_select,wheel_index,:));
        end
        
        
        %thrust force [N]
        temp_xi = ones(length(soil_select),1)*Wheel.slip_range;
        temp_yi = squeeze(ContactAngle(soil_select,wheel_index,:));
        temp_zi = squeeze(UnloadSink(soil_select,wheel_index,:));
        switch length(soil_select)
            case 1
                temp_xi = reshape(temp_xi,1,length(temp_xi));
                temp_yi = reshape(temp_yi,1,length(temp_yi));
                temp_zi = reshape(temp_zi,1,length(temp_zi));
        end
        ThrustForce_temp(soil_select,:) = nanmodified_interp3(...
            Wheel.slip_range, Wheel.thetaf_range, Wheel.unloadsink_range, ThrustLookup_temp,...
            temp_xi, temp_yi, temp_zi);

        
        %resistance force [N]
        temp_xi = ones(length(soil_select),1)*Wheel.slip_range;
        temp_yi = squeeze(ContactAngle(soil_select,wheel_index,:));
        temp_zi = squeeze(UnloadSink(soil_select,wheel_index,:));
        switch length(soil_select)
            case 1
                temp_xi = reshape(temp_xi,1,length(temp_xi));
                temp_yi = reshape(temp_yi,1,length(temp_yi));
                temp_zi = reshape(temp_zi,1,length(temp_zi));
        end
        ResistForce_temp(soil_select,:) = nanmodified_interp3(...
            Wheel.slip_range, Wheel.thetaf_range, Wheel.unloadsink_range, ResistLookup_temp,...
            temp_xi, temp_yi, temp_zi);

        
        %combine force from all wheels (2*temp to factor left/right wheels)
        ThrustForce(soil_select,:) = ThrustForce(soil_select,:) + 2*ThrustForce_temp(soil_select,:);
        ResistForce(soil_select,:) = ResistForce(soil_select,:) + 2*ResistForce_temp(soil_select,:);
        
    end

end
    

%drawbar pull: subtract desired force from wheel force (thrust-resist) [N]
DrawbarPull = ThrustForce-ResistForce-Dforce_temp*ones(size(Dforce_temp,2),size(ThrustForce,2));

%find maximum drawbar pull (per operating condition and mass) [N]
MaxDrawbarPull = max(DrawbarPull,[],2);

%interpolate slip to find zero drawbar pull case (meets spec)
temp(:,1) = userdefined_interpindex(DrawbarPull,0);
WheelSlip=nan(size(DrawbarPull,1),1);
WheelSlip(~isnan(temp),1) = interp1(Wheel.slip_range,temp(~isnan(temp),1));

%find thrust and resistance forces at slip [N]
switch size(ThrustForce,1)
    case 1
        ThrustForce_slip(:,1) = nanmodified_interp1(1:size(ThrustForce,2),ThrustForce,temp);
        ResistForce_slip(:,1) = nanmodified_interp1(1:size(ResistForce,2),ResistForce,temp);
    otherwise
        ThrustForce_slip(:,1) = nanmodified_interp2(1:size(ThrustForce,2),1:size(ThrustForce,1),ThrustForce,temp,(1:size(ThrustForce,1))');
        ResistForce_slip(:,1) = nanmodified_interp2(1:size(ResistForce,2),1:size(ResistForce,1),ResistForce,temp,(1:size(ResistForce,1))');
end



%% Wheel Speed and Power

%find wheel speed for positive and negative slip cases [rad/s]
                    %accel>=0                                accel<0
WheelSpeeds = [Speed_temp./(Vehicle.r.*(1-WheelSlip)) (1/Vehicle.r).*Speed_temp.*(1+WheelSlip)];

%find sign of acceleration
AccelSign = [Accel_temp>=0 Accel_temp<0];

%wheel speed [rad/s]
WheelSpeed(:,1) = WheelSpeeds(:,1).*AccelSign(:,1) + WheelSpeeds(:,2).*AccelSign(:,2);

%wheel torque required (input to wheel) and output (from wheel) [N-m]
Torque_Required(:,1) = Vehicle.r*(ThrustForce_slip+ResistForce_slip);
Torque_Output(:,1) = Vehicle.r*(ThrustForce_slip-ResistForce_slip);


%wheel power [N-m/s or Watt]
Power_Required(:,1) = WheelSpeed.*Torque_Required;
Power_Output(:,1) = WheelSpeed.*Torque_Output;


%% Feasibility
WheelFeasibility(:,1) = WheelFeasibility(:,1).*(~isnan(WheelSlip));



%% Outputs
Output.WheelFeasibility=WheelFeasibility;

Output.WheelSlip=WheelSlip;
Output.WheelSpeed=WheelSpeed;

Output.Thrust=ThrustForce_slip;
Output.Resist=ResistForce_slip;

Output.MaxDrawbarPull=MaxDrawbarPull;

Output.TorqueRequired=Torque_Required;
Output.PowerRequired=Power_Required;
Output.PowerOutput=Power_Output;


