%Description: create tables of terramechanics input/output relationship for future use
%inputs: 
    % Wheel radius range, width range, lug (grouser) height and width, material density, volume
    % List of soil types (Wheel.TerrainList)
%outputs:
    % wheel_r_radius_b_width.mat file to Wheel_Database folder
    % .mat file includes input/output tables (e.g. normal load, thrust force)

%% Wheel Parameters
rrange=0.12:0.02:0.16;
brange=0.04:0.02:0.16;
for rindex=1:length(rrange)
for bindex=1:length(brange)    

disp(['radius = ',num2str(rrange(rindex)),'     width = ',num2str(brange(bindex))]);
    
Wheel.r = rrange(rindex); %radius of wheels [m]
Wheel.b = brange(bindex); %width of wheels [m]
Wheel.mu = 1; %ratio of lug to concave portion, 1=no lugs
Wheel.lh = 0; %height of lug [m]

Wheel.density = 2.6989*(100^3/1000); %aluminum=2.6989 g/cc, convert to kg/m^3
Wheel.volume = (Wheel.r^2-(0.95*Wheel.r)^2)*pi*Wheel.b; %assume thick walled cylindrical tube
Wheel.mass = Wheel.density*Wheel.volume;
Wheel.inertia = 0.5*Wheel.mass*(Wheel.r^2+(0.95*Wheel.r)^2);

%% slip range must be symmetric %%
% temp_slipresolution = 35;
temp_slipresolution = 25;
temp_negslip=linspace(-0.95,0,temp_slipresolution);
temp_posslip=linspace(-temp_negslip(end-1),max(-temp_negslip),temp_slipresolution-1);
Wheel.slip_range = [temp_negslip,temp_posslip];

Wheel.TerrainList = {'LooseAirDrySand', 'MediumSoil', 'ClayedSoil', 'UplandSandyLoam'};
%choices: 'LooseAirDrySand', 'MediumSoil', 'ClayedSoil', 'UplandSandyLoam'

%% front contact angle range
% Wheel.thetaf_range = (0:0.5:90).*(pi/180);
Wheel.thetaf_range = (0:1:90).*(pi/180);
temp_maxsink=Wheel.r*(1-cos(max(Wheel.thetaf_range))); %maximum sinkage of 1st wheel
temp_numberaxles = 4; %assume no more than 4 axles (4 wheel per line)

% Wheel.unloadsink_range = linspace(0,temp_numberaxles*temp_maxsink,temp_numberaxles*10);

%Wheel.unloadsink_range does not need to be linear necessarily - in fact it is
%best if we concentrate at low sinkage values, where we are more likely to
%be (unlikely to ever reach max sinkage)

%try using third order function:
temp_linrange=linspace(0,(temp_numberaxles*temp_maxsink)^(1/3),10);
Wheel.unloadsink_range=temp_linrange.^3;

%% Terrain Parameters
for terrain_index=1:3%length(Wheel.TerrainList)

disp(['terrain = ',num2str(terrain_index)]);
    
terrain_type = Wheel.TerrainList{terrain_index}; 
     
Terrain = terraintype(terrain_type);


%% Build Lookup Tables
eval(['[Wheel.',terrain_type,'.PosNormalLookup,Wheel.',terrain_type,'.PosForceLookup,Wheel.',terrain_type,'.PosResistLookup,Wheel.',terrain_type,'.NormalForceRange,Wheel.',terrain_type,'.ContactAngleLookup,Wheel.',terrain_type,'.MaxSinkageLookup] = buildTable(Wheel,Terrain,Wheel.slip_range,Wheel.thetaf_range,Wheel.unloadsink_range);'])


end

%Name based on wheel configuration
self_ChangeDir('Wheel_Database'); %change directory to saved folder
savefile = ['wheel_r_',num2str(Wheel.r*1000),'_b_',num2str(Wheel.b*1000),'.mat'];
save(savefile,'Wheel')
self_ChangeDir(); %move up one folder

end
end