%description: 3-D table linear interpolation (interp3) modified to handle nan

function vi = nanmodified_interp3(varargin)

error(nargchk(1,9,nargin,'struct')); % allowing for an ExtrapVal

ExtrapVal = nan; % protecting default

% interp3(x,y,z,v,xi,yi,zi)
[msg,x,y,z,v,xi,yi,zi] = xyzvchk(varargin{1:7});


%
% Check for non-equally spaced data.  If so, map (x,y) and
% (xi,yi) to matrix (row,col) coordinate system.
%

  xx = x(1,:,1).'; yy = y(:,1,1); zz = squeeze(z(1,1,:)); % columns
  dx = diff(xx); dy = diff(yy); dz = diff(zz);
  xdiff = max(abs(diff(dx))); if isempty(xdiff), xdiff = 0; end
  ydiff = max(abs(diff(dy))); if isempty(ydiff), ydiff = 0; end
  zdiff = max(abs(diff(dz))); if isempty(zdiff), zdiff = 0; end
  if (xdiff > eps*max(xx)) || (ydiff > eps*max(yy)) || (zdiff > eps*max(zz))
    if any(dx < 0), % Flip orientation of data so x is increasing.
      x = flipdim(x,2); y = flipdim(y,2); 
      z = flipdim(z,2); v = flipdim(v,2);
      xx = flipud(xx); dx = -flipud(dx);
    end
    if any(dy < 0), % Flip orientation of data so y is increasing.
      x = flipdim(x,1); y = flipdim(y,1);
      z = flipdim(z,1); v = flipdim(v,1);
      yy = flipud(yy); dy = -flipud(dy);
    end
    if any(dz < 0), % Flip orientation of data so y is increasing.
      x = flipdim(x,3); y = flipdim(y,3);
      z = flipdim(z,3); v = flipdim(v,3);
      zz = flipud(zz); dz = -flipud(dz);
    end
  
      % Determine the nearest location of xi in x
      [xxi,j] = sort(xi(:));
      [ignore,i] = sort([xx;xxi]);
      si(i) = (1:length(i));
      si = (si(length(xx)+1:end)-(1:length(xxi)))';
      si(j) = si;
    
      % Map values in xi to index offset (si) via linear interpolation
      si(si<1) = 1;
      si(si>length(xx)-1) = length(xx)-1;
      si = si + (xi(:)-xx(si))./(xx(si+1)-xx(si));
     
      % Determine the nearest location of yi in y
      [yyi,j] = sort(yi(:));
      [ignore,i] = sort([yy;yyi]);
      ti(i) = (1:length(i));
      ti = (ti(length(yy)+1:end)-(1:length(yyi)))';
      ti(j) = ti;
    
      % Map values in yi to index offset (ti) via linear interpolation
      ti(ti<1) = 1;
      ti(ti>length(yy)-1) = length(yy)-1;
      ti = ti + (yi(:)-yy(ti))./(yy(ti+1)-yy(ti));
    
      % Determine the nearest location of zi in z
      [zzi,j] = sort(zi(:));
      [ignore,i] = sort([zz;zzi]);
      wi(i) = (1:length(i));
      wi = (wi(length(zz)+1:end)-(1:length(zzi)))';
      wi(j) = wi;
    
      % Map values in zi to index offset (wi) via linear interpolation
      wi(wi<1) = 1;
      wi(wi>length(zz)-1) = length(zz)-1;
      wi = wi + (zi(:)-zz(wi))./(zz(wi+1)-zz(wi));
    
      [x,y,z] = meshgrid(ones(class(x)):size(x,2),...
                         ones(superiorfloat(y,z)):size(y,1),1:size(z,3));
      xi(:) = si; yi(:) = ti; zi(:) = wi;

  end


% Now do the interpolation
vi = linear(x,y,z,v,xi,yi,zi,ExtrapVal);


%------------------------------------------------------
function F = linear(arg1,arg2,arg3,arg4,arg5,arg6,arg7,ExtrapVal)

% linear(x,y,z,v,s,t,w), X, Y and Z specified.
[nrows,ncols,npages] = size(arg4);
mx = numel(arg1); my = numel(arg2); mz = numel(arg3);

s = 1 + (arg5-arg1(1))/(arg1(mx)-arg1(1))*(ncols-1);
t = 1 + (arg6-arg2(1))/(arg2(my)-arg2(1))*(nrows-1);
w = 1 + (arg7-arg3(1))/(arg3(mz)-arg3(1))*(npages-1);
  

% Check for out of range values of s and set to 1
sout = find((s<1)|(s>ncols));
if ~isempty(sout), s(sout) = ones(size(sout)); end

% Check for out of range values of t and set to 1
tout = find((t<1)|(t>nrows));
if ~isempty(tout), t(tout) = ones(size(tout)); end

% Check for out of range values of w and set to 1
wout = find((w<1)|(w>npages));
if ~isempty(wout), w(wout) = ones(size(wout)); end

% Matrix element indexing
nw = nrows*ncols;
ndx = floor(t)+floor(s-1)*nrows+floor(w-1)*nw;

% Compute intepolation parameters, check for boundary value.
if isempty(s), d = s; else d = find(s==ncols); end
s(:) = (s - floor(s));
if ~isempty(d), s(d) = s(d)+1; ndx(d) = ndx(d)-nrows; end

% Compute intepolation parameters, check for boundary value.
if isempty(t), d = t; else d = find(t==nrows); end
t(:) = (t - floor(t));
if ~isempty(d), t(d) = t(d)+1; ndx(d) = ndx(d)-1; end

% Compute intepolation parameters, check for boundary value.
if isempty(w), d = w; else d = find(w==npages); end
w(:) = (w - floor(w));
if ~isempty(d), w(d) = w(d)+1; ndx(d) = ndx(d)-nw; end


% Now interpolate.
F=nan(size(ndx));
ok_index=find(~isnan(ndx));


F(ok_index) =  (( arg4(ndx(ok_index)).*(1-t(ok_index)) + arg4(ndx(ok_index)+1).*t(ok_index) ).*(1-s(ok_index)) + ...
    ( arg4(ndx(ok_index)+nrows).*(1-t(ok_index)) + arg4(ndx(ok_index)+(nrows+1)).*t(ok_index) ).*s(ok_index)).*(1-w(ok_index)) +...
   (( arg4(ndx(ok_index)+nw).*(1-t(ok_index)) + arg4(ndx(ok_index)+1+nw).*t(ok_index) ).*(1-s(ok_index)) + ...
    ( arg4(ndx(ok_index)+nrows+nw).*(1-t(ok_index)) + arg4(ndx(ok_index)+(nrows+1+nw)).*t(ok_index) ).*s(ok_index)).*w(ok_index);
    
%   F =  (( arg4(ndx).*(1-t) + arg4(ndx+1).*t ).*(1-s) + ...
%         ( arg4(ndx+nrows).*(1-t) + arg4(ndx+(nrows+1)).*t ).*s).*(1-w) +...
%        (( arg4(ndx+nw).*(1-t) + arg4(ndx+1+nw).*t ).*(1-s) + ...
%         ( arg4(ndx+nrows+nw).*(1-t) + arg4(ndx+(nrows+1+nw)).*t ).*s).*w;


% Now set out of range values to ExtrapVal.
if ~isempty(sout), F(sout) = ExtrapVal; end
if ~isempty(tout), F(tout) = ExtrapVal; end
if ~isempty(wout), F(wout) = ExtrapVal; end



